/*
    Copyright (c) 2006 Volker Krause <vkrause@kde.org>
    Copyright (c) 2007 Robert Zwerus <arzie@dds.nl>

    This library is free software; you can redistribute it and/or modify it
    under the terms of the GNU Library General Public License as published by
    the Free Software Foundation; either version 2 of the License, or (at your
    option) any later version.

    This library is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
    License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to the
    Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
    02110-1301, USA.
*/

#ifndef ITEMDUMPER_H
#define ITEMDUMPER_H

#include <akonadi/collection.h>

#include <QTime>

class KJob;

class ItemDumper : public QObject
{
  Q_OBJECT
  public:
    ItemDumper( const QString &path, const QString &filename, const QString &mimetype, int count );

  private Q_SLOTS:
    void done(KJob* job);

  private:
    QTime mTime;
    int mJobCount;

};




#endif
